# RY352A_Position_Datetime_more_Display.py
# 31.01.2024
# Das Programm ermittelt GNSS-Daten mit Hilfe des Moduls RY352A von REYAX
# und zeigt sie auf dem Display an.


# Folgende Datensätze werden angezeigt:

# Position: 51° 10.6416' N ,  006° 28.3447' E (aus $GNMRC)
# Zeit: 10:29:54, 19 Satelliten
# Höhe (Geoid): 47.5 m (aus $GNGGA), Geschwindigkeit über Grund: 0.05 km/h (aus $GNVTG)

####################################################################################
# Das Programm benutzt die Klasse RYS352A:                                         #
# gnss = RY352A(uart, ticks_ms) instanziiert das Objekt gnss.                      #
#                                                                                  #
# gnss besitzt die Methoden read_item, get_data und send_cmd                       #
#                                                                                  #                                                                                  #
# gnss.read_item() liest einen Datensatz (Zeichenkette) aus dem Lesepuffer.        #
#                                                                                  #
# gnss.get_data(data_name) liefert als Rückgabewert eine Liste mit dem Datensatz   #
# der in der Zeichenkette data_name angebenen Kennung (ohne die Kennung selbst).   #
# gnss.get_data('$GPRMC') liefert z. B. Zeit und Datum, Längen- und Breitengrad    #
# von "GPS", gnss.get_data('$GNRMC') die entsrechenden Daten von einer             #
# Kombination mehrerer Systeme (z. B. GPS und GLONASS). In dem folgenden Programm  #
# wird immer mit einer Kombination von GPS und GLONASS gearbeitet.                 #
# Ist ein Fehler bei der Ausführung dieser Methode aufgetreten,                    #
# wird eine leere Liste zurückgegeben.                                             #
# Wenn ein Datensatz (z. B. $GNXXX) nicht gefunden werden kann, wird auf dem       #
# Terminal eine eine Fehlermeldung ausgegben; die zugehörige Liste ist dann leer.  #
#                                                                                  #
# gnss.send_cmd('$PAIR051*3E') sendet das PAIR Command $PAIR051 an das             #
# RYS352A-Modul.                                                                   #
####################################################################################


####################################
# Anschlüsse:                      #
# RY352A-Modul       --  TTGO      # 
# VCC (5:rot)        --  3V        #
# GND (2:schwarz)    --  G         #
# TX (3: grün)       --  13 (RX)   #
# RX (4: weiß)       --  12 (TX)   #
####################################


from machine import UART
from time import sleep
from utime import ticks_ms
from machine import Pin, SPI
import vga1_16x16 as font1
import vga2_8x16 as font2
import st7789
from rys352a import RYS352A

# Instanziierungen/Initialisierungen:
uart = UART(1, baudrate=115_200, bits=8, parity=None, stop=1, tx=12, rx=13, rts=-1, cts=-1, txbuf=256, rxbuf=256, timeout=5000, timeout_char=2)

id = 1 # id = 2 # z. B. für Firmware V 1.12 und V 1.14, id = 1 für V= 1.18 und V.1.20
spi = SPI(id, baudrate=20_000_000, polarity=1, sck=Pin(18), mosi=Pin(19))
display = st7789.ST7789(spi, 135, 240,  reset=Pin(23, Pin.OUT), cs=Pin(5, Pin.OUT), dc=Pin(16, Pin.OUT), backlight=Pin(4, Pin.OUT), rotation=3) # Landscape
display.init()
display.fill(0)
display.rect(0,0,239,24,st7789.WHITE) # Titel mit Rahmen
display.text(font1, 'TTGO-GNNS', 48, 5) 

gnss = RYS352A(uart, ticks_ms) # gps-Objekt instanziieren

        
################################## Formatierungen #########################################

# Bei den folgenden Funktionen sind die Rückgabewerte vom Typ Bytes,
# weil Sonderzeichen benutzt werden. display.text akzeptiert (normale) Strings
# als auch Byte-Strings.

def N_S_to_degree(s) -> bytes: # Angaben für N/S in Grad umwandeln
    v1 = s[0:2]
    v2 = s[2:4]
    v3 = s[5:9]
    return bytes(v1, 'UTF-8')+b'\xF8'+bytes(v2+'.'+v3+chr(39), 'UTF-8')

def E_W_to_degree(s) -> bytes: # Angaben für E/W in Grad umwandeln
    v1 = s[0:3]
    v2 = s[3:5]
    v3 = s[6:10]
    return bytes(v1, 'UTF-8')+b'\xF8'+bytes(v2+'.'+v3+chr(39), 'UTF-8')

def time_to_h_min_s(s) -> bytes: # Angaben in hh:mm:ss umwandeln
    v1 = s[0:2]
    v2 = s[2:4]
    v3 = s[4:6]
    return v1+':'+v2+':'+v3

################################ Hauptprogramm #################################

print('Programm gestartet...')
print()

while True:
    # XXX_list = gnss.get_data('$GNXXX') # falscher $GN-Code zum Testen von Timeout
    # print(XXX_list)
    RMC_list = gnss.get_data('$GNRMC') # '$GPRMC', wenn nur GPS
    # print(RMC_list)
    GGA_list = gnss.get_data('$GNGGA') # '$GPGGA', wenn nur GPS
    # print(GGA_list)   
    VTG_list = gnss.get_data('$GNVTG') # '$GPVTG', wenn nur GPS
    # print(VTG_list)
    # print(gnss.read_item()) # read_item testen

    if RMC_list == [] or GGA_list == [] or VTG_list == []:
        print('Nicht genügend Daten empfangen...')
        break

    # Block 1 (Koordinaten)
    display.rect(0, 35, 239, 24, st7789.RED)
    display.text(font2,N_S_to_degree(RMC_list[2]),5,40)
    display.text(font2, RMC_list[3], 95, 40)
    display.text(font2,E_W_to_degree(RMC_list[4]),130,40)
    display.text(font2, RMC_list[5], 225, 40)    

    # Block 2 (Uhrzeit, Anzahl der Satelliten)
    display.rect(0, 70, 239, 24, st7789.BLUE)
    display.text(font2, 'Zeit: ' + time_to_h_min_s(GGA_list[0]), 5, 75)
    display.text(font2, 'Anz. Sat.: ' + GGA_list[6], 130, 75)    

    # Block 3 (Höhe bzgl. Geoid, Geschwindigkeit über Grund)
    display.rect(0, 105, 239, 24, st7789.GREEN)
    display.text(font2, 'h(Geo): ' + GGA_list[8] + ' m', 5, 110)
    display.text(font2, 'v: ' + VTG_list[6] + ' km/h', 130, 110)

    sleep(2)
    

    






